 
%{                                          /* -*- C++ -*- */
#include "ValueExpressionDriver.H"
#include "ValueExpressionParser.tab.hh"
#include <errno.h>
%}

%option noyywrap nounput batch debug 
%option stack
%option prefix="ve"

id      [[:alpha:]_][[:alnum:]_]*
int     [[:digit:]]+

exponent_part              [eE][-+]?[[:digit:]]+
fractional_constant        [-]?(([[:digit:]]*"."[[:digit:]]+)|([[:digit:]]+".")|([[:digit:]]))
float                      ((({fractional_constant}{exponent_part}?)|([[:digit:]]+{exponent_part}))|0)

%{
# define YY_USER_ACTION yylloc->columns (yyleng);
%}
%%

%{
    yylloc->step ();
%}

[ \t]+             yylloc->step ();
[\n]+                yylloc->lines (yyleng); yylloc->step ();

[-+*/(),&^<>!?:.]               return yytext[0];

&&                   return TOKEN_AND;
\|\|                   return TOKEN_OR;
==                   return TOKEN_EQ;
!=                   return TOKEN_NEQ;
\<=                   return TOKEN_LEQ;
\>=                   return TOKEN_GEQ;

pow                   return TOKEN_pow;
exp                   return TOKEN_exp;
log                   return TOKEN_log;
mag                   return TOKEN_mag;
sin                   return TOKEN_sin;
cos                   return TOKEN_cos;
tan                   return TOKEN_tan;
min                   return TOKEN_min;
max                   return TOKEN_max;
sqr                   return TOKEN_sqr;
sqrt                  return TOKEN_sqrt;

pi                    return TOKEN_pi;
pos                   return TOKEN_position;
fpos                  return TOKEN_fposition;
face                  return TOKEN_face;
area                  return TOKEN_area;
vol                   return TOKEN_volume;
dist                  return TOKEN_dist;
rand                  return TOKEN_rand;
randNormal            return TOKEN_randNormal;
set                   return TOKEN_set;

grad                  return TOKEN_grad;
curl                  return TOKEN_curl;
magSqrGradGrad        return TOKEN_magSqrGradGrad;
snGrad                return TOKEN_snGrad;
div                   return TOKEN_div;
laplacian             return TOKEN_laplacian;

integrate             return TOKEN_integrate;
sum                   return TOKEN_sum;
interpolate           return TOKEN_interpolate;
average               return TOKEN_average;
reconstruct           return TOKEN_reconstruct;

vector                 return TOKEN_VECTOR;

true                   return TOKEN_TRUE;
false                  return TOKEN_FALSE;

{float}                {
                       errno = 0;
                       yylval->val = atof(yytext);
                       return TOKEN_NUM;
                     }

[xyz]                return yytext[0];

{id}                 {
    Foam::string *ptr=new Foam::string (yytext);
    if(driver.isVector(*ptr)) {
        yylval->vname = ptr; return TOKEN_VID;
    } else if(driver.isScalar(*ptr)) {
        yylval->name = ptr; return TOKEN_SID;
    } else if(driver.isFaceScalar(*ptr)) {
        yylval->name = ptr; return TOKEN_FSID;
    } else if(driver.isFaceVector(*ptr)) {
        yylval->name = ptr; return TOKEN_FVID;
    } else if(driver.isCellSet(*ptr)) {
        yylval->name = ptr; return TOKEN_SETID;
    } else {
        driver.error (*yylloc, "field "+*ptr+" not existing or of wrong type");
    }
                     }

.                    driver.error (*yylloc, "invalid character");


%%

void ValueExpressionDriver::scan_begin ()
{
    yy_flex_debug = trace_scanning;
    yy_scan_string(content.c_str());
//    if (!(yyin = fopen (file.c_str (), "r")))
//        error (std::string ("cannot open ") + file);
}

void ValueExpressionDriver::scan_end ()
{
//	    fclose (yyin);
}
